require 'open-uri'
require 'httparty'

class Shortener

  def process(item, clicked)
    begin
      url = URI.parse(item)
      url = URI.parse("http://" + item) unless url.scheme
    rescue URI::InvalidURIError
      if (clicked)
        $dz.fail("No valid URL on clipboard to shorten")
      else
        $dz.fail("Invalid URL")
      end
    end

    url_as_string = url.to_s

    if url_as_string =~ /https?:\/\/tinyurl\.com\//
      $dz.begin('Getting expanded URL')
      expand_url(url_as_string)
    else
      if (clicked)
        config = "
        *.title = Confirm Shorten URL
        cb.type = cancelbutton
        txt.type = text
        txt.default = Shorten '#{url_as_string}' using TinyURL?
        "
        result = $dz.pashua(config)

        if result['cb'] == "1"
          $dz.fail("Cancelled")
        end
      end
      
      $dz.begin('Getting shortened URL')
      shorten_url(url_as_string)
    end
  end

  def shorten_url(url_to_shorten)
    error_shown = false
    
    begin
      if (ENV['api_token'] and ENV['api_token'].strip != '')
        result = HTTParty.post("https://api.tinyurl.com/create", :headers => { "Authorization" => "Bearer #{ENV['api_token']}", "accept" => "application/json", "Content-Type" => "application/json" }, :body => {:url => url_to_shorten}.to_json)

        if (result['code'] == 0)
          # OK
          short_url = result['data']['tiny_url']
        elsif (result['code'] == 1)
          error_shown = true
          $dz.error('Failed to shorten URL', "TinyURL Authorization failed.\nCheck the API Token in the action options is set correctly.")
        else
          # Should be errors in error array
          if result['errors'] and result['errors'].length
            error_shown = true
            $dz.error('Failed to shorten URL', result['errors'][0])
          else
            # Unknow error, just show the code
            error_shown = true
            $dz.error('Failed to shorten URL', "TinyURL returned error code: #{result['code']}")
          end
        end
      else
        short_url = open('https://tinyurl.com/api-create.php?url=' + url_to_shorten, "UserAgent" => "Ruby Script").read
      end
    rescue Exception => e
      $dz.error('Failed to shorten URL', e.message) unless error_shown
    end
    
    if short_url.to_s.strip.length == 0
      $dz.fail('Error shortening URL')
    else
      $dz.finish("URL is now on clipboard")
      $dz.url(short_url)
    end
  end

  def expand_url(url_to_expand)
    begin
      uri = URI(url_to_expand)
      expanded_url = Net::HTTP.new(uri.host, 80).get(uri.path).header['location']
    rescue Exception => e
      $dz.error('Failed to expand URL', e.message)
    end
    
    if expanded_url.to_s.strip.length == 0
      $dz.fail('Error expanding URL')
    else
      $dz.finish("Expanded URL is now on clipboard")
      $dz.url(expanded_url)
    end
  end
end